/* output format: default, bit endian, litte endian */
OUTPUT_FORMAT("elf32-littlearm", "elf32-bigarm", "elf32-littlearm")
/* output architecture */
OUTPUT_ARCH(arm)
/* memory layout */
MEMORY {
	/* flash: origins at 0x0 with 32MiB size */
	FLASH	(r!wx)	: ORIGIN = 0x00000000, LENGTH = 32M
	/* sram: origins at 0x5c000000 with 256KiB size */
	SRAM	(rwx)	: ORIGIN = 0x5c000000, LENGTH = 256K
	/* sdram: origins at 0xA0000000 with 32MiB size */
	SDRAM	(rwx)	: ORIGIN = 0xA0000000, LENGTH = 32M
}
/* section mapping */
SECTIONS {
	/* image start in FLASH memory */
	__image_start = 0x00400000;
	
	/* text section starts at image start */
	.text __image_start :
	{
		/* vectors */
		*(.vectors)
		
		/* absolute vector addresses */
		. = 0x20;
		*(.vecaddrs)
		
		/* image header */
		. = 0x40;
		*(.header)
		
		/* image loader */
		*(.loader)
		
		/* the text section, rodata section, ... */
		__text_start = .;
		/* text input sections */
		*(.text .stub .text.* .gnu.linkonce.t.*)
		/* read only data */
		*(.rodata.* .gnu.linkonce.r.*)
		*(.rodata)
		/* TODO: do we really need that section? */
		/**(.comment)*/
		/* .gnu.warning sections are handled specially by elf32.em. */
		*(.gnu.warning)
		*(.glue_7t) *(.glue_7)
		/* exception table */
		*(.gcc_except_table)
	} >FLASH
	
	/* init section, align to 4 byte boundary */
	.init ALIGN(4) :
	{
		/* keep init sections of crti files */
		KEEP(*crti.o (.init))
		/* keep init sections of all files */
		KEEP(*(.init))
	} >FLASH
	
	.fini ALIGN(4) :
	{
		/* keep finalize sections of crt files */
		KEEP(*crti.o (.fini))
		/* keep finalize sections */
		KEEP (*(.fini))
	} >FLASH
	
	/* constructors, align to 4 byte boundary */
	.ctors ALIGN(4) :
	{
		/* provide start of constructors symbol */
		PROVIDE(__ctors_start__ = .);
		/* keep ctors of crtbegin files */
		KEEP(*crtbegin*.o (.ctors))
		/* exclude ctors of crtend files */
		KEEP(*(EXCLUDE_FILE (*crtend*.o) .ctors))
		/* keep sorted ctors of all files */
		KEEP(*(SORT(.ctors.*)))
		/* keep ctors of all files */
		KEEP(*(.ctors))
		/* provide end of constructors symbol */
		PROVIDE(__ctors_end__ = .);
	} >FLASH
	
	/* destructors, align to 4 byte boundary */
	.dtors ALIGN(4) :
	{
		/* provide start of destructors symbol */ 
		PROVIDE(__dtors_start__ = .);
		/* keep dtors of crtbegin files */
		KEEP(*crtbegin*.o (.dtors))
		/* exclude dtors of crtend files */
		KEEP(*(EXCLUDE_FILE (*crtend*.o) .dtors))
		/* keep sorted dtors of all files */
		KEEP(*(SORT(.dtors.*)))
		/* keep dtors of all files */
		KEEP(*(.dtors))
		/* provide end of destructors symbol */
		PROVIDE(__dtors_end__ = .);
	} >FLASH
	
	/* jcr section, align to 4 byte boundary */
	.jcr ALIGN(4) :
	{
		/* keep jcr section of all files */
		KEEP(*(.jcr))
	} >FLASH

	/* eh_frame section, align to 4 byte boundary */
	.eh_frame ALIGN(4) :
	{
		/* keep eh_frame section of all files */
		KEEP(*(.eh_frame))
	} >FLASH
	
	/* end of text section and text size */
	__text_end = .;
	__text_size = __text_end - __text_start;
	
	/* data section, loaded into ROM right after the above sections, but is
		copied into memory at the beginning of SRAM */
	.data :
	{
		/* provide data start symbol */
		__data_start = .;
		/* vector table */
		*(.vectable)
		*(.ramtext)
		/* data sections */
		*(.data .data.* .gnu.linkonce.d.*)
		*(.gnu.linkonce.d*)
		/* provide data end symbol */
		__data_end = .;
		/* magic at end of data section, denoting end of image */
		LONG(0xdeadc0de);
	} >SRAM AT>FLASH
	
	/* provide image size symbol
		difference of LMA of data section plus its size
		and the LMA(=VMA) of text section
		see also magic.s */
	__image_size = ABSOLUTE(LOADADDR(.data) + SIZEOF(.data)) - ADDR(.text);
	
	/* the following symbols are needed for the loader to get the data
	 * section into the right location (from its LMA to its VMA) */
	/* provide load address of data section */
	__data_load_start = LOADADDR(.data);
	/* and the size */
	__data_size = __data_end - __data_start;
	
	/* uninitialized data section, initialized with zeros.
		located after data section in SRAM */
	.bss (NOLOAD) :
	{
		/* provide bss start symbol */
		__bss_start = .;
		/* bss sections */
		*(.dynbss)
		*(.bss .bss.* .gnu.linkonce.b.*)
		*(COMMON)
		/* provide bss end symbol */
		__bss_end = .;
	} >SRAM
	
	__bss_size = __bss_end - __bss_start;
	
	.heap (NOLOAD) :
	{
		/* provide end symbol, after which malloc() and friends allocate
		 * memory dynamically until they reach the stack pointer */
		end = .;
	} >SRAM
	
	/* Stack sizes of System modes (FIQ, IRQ, SVC, ABT, UND) */
	_FIQ_STACK_SIZE = 400;
	_IRQ_STACK_SIZE = 2k;
	_SVC_STACK_SIZE = 3k;
	_ABT_STACK_SIZE = 1k;
	_UND_STACK_SIZE = 1k;
	
	/* Provide the end of stack symbol. As stack grows downwards, this is
	 * the initial adress for the stack pointers. See also stack.s */
	_STACK_END = ORIGIN(SRAM) + LENGTH(SRAM);
	
	
	/* Stabs debugging sections.  */
	.stab          0 : { *(.stab) }
	.stabstr       0 : { *(.stabstr) }
	.stab.excl     0 : { *(.stab.excl) }
	.stab.exclstr  0 : { *(.stab.exclstr) }
	.stab.index    0 : { *(.stab.index) }
	.stab.indexstr 0 : { *(.stab.indexstr) }
	.comment       0 : { *(.comment) }
	/* DWARF debug sections.
		Symbols in the DWARF debugging sections are relative to the beginning
		of the section so we begin them at 0.  */
	/* DWARF 1 */
	.debug          0 : { *(.debug) }
	.line           0 : { *(.line) }
	/* GNU DWARF 1 extensions */
	.debug_srcinfo  0 : { *(.debug_srcinfo) }
	.debug_sfnames  0 : { *(.debug_sfnames) }
	/* DWARF 1.1 and DWARF 2 */
	.debug_aranges  0 : { *(.debug_aranges) }
	.debug_pubnames 0 : { *(.debug_pubnames) }
	/* DWARF 2 */
	.debug_info     0 : { *(.debug_info .gnu.linkonce.wi.*) }
	.debug_abbrev   0 : { *(.debug_abbrev) }
	.debug_line     0 : { *(.debug_line) }
	.debug_frame    0 : { *(.debug_frame) }
	.debug_str      0 : { *(.debug_str) }
	.debug_loc      0 : { *(.debug_loc) }
	.debug_macinfo  0 : { *(.debug_macinfo) }
	/* SGI/MIPS DWARF 2 extensions */
	.debug_weaknames 0 : { *(.debug_weaknames) }
	.debug_funcnames 0 : { *(.debug_funcnames) }
	.debug_typenames 0 : { *(.debug_typenames) }
	.debug_varnames  0 : { *(.debug_varnames) }
	/DISCARD/ : { *(.note.GNU-stack) }
}
