#include "registers.h"
#include "pmic.h"
#include "hardware.h"
#include "interrupt.h"

static inline void initPMIC() {
	CKEN |=  CKEN_CKEN15;
	// I²C bus on, current change on frequency change on, keep DC/DC converter running in sleep
	PCFR = PCFR_PI2C_EN | PCFR_FVC | PCFR_DC_EN;
	PICR = ICR_IUE | ICR_SCLE;
	GAFR3_L &= ~(3 << 24);
	GPDR3 |= (1<<12); // Direction Register auf Output
	GPSR3 &= ~(1<<12); // Set Register setzen
	writePMIC(PMIC_SYS_CONTROL_A,SCA_RESET_WDOG | SCA_WDOG_ACTION | SCA_HWRES_EN);
	writePMIC(PMIC_IRQ_MASK_A, ~IMA_ONKEY_N);
	writePMIC(PMIC_IRQ_MASK_B, 0xFF);
	writePMIC(PMIC_IRQ_MASK_C, 0xFF);
	//read out the EVENT registers so that we can receive interrupts
	readPMIC(PMIC_EVENTA);
	readPMIC(PMIC_EVENTB);
	readPMIC(PMIC_EVENTC);
	
	/* FIXME: configure pmcr properly */
	PMCR = PMCR_BIDAE | PMCR_VIDAE;
}

/*Daten an den PMIC schicken
 */
void writePMIC(const uint8_t address, const uint8_t value) {
	PICR = ICR_IUE | ICR_SCLE;
	PIDBR = PMIC_SLAVE_ADDR<<1;
	PICR |= ICR_START | ICR_TB;
	while(PICR & ICR_TB);
	
	PIDBR = address;
	PICR &= ~ICR_START;
	PICR |= ICR_TB;
	while(PICR & ICR_TB);
	
	PIDBR = value;
	PICR |= ICR_STOP | ICR_TB;
	while(PICR & ICR_TB);
	PICR &= ~ICR_STOP;
}

/*einzelnes Datum des PMIC lesen
 */
uint8_t readPMIC(const uint8_t address) {
	uint8_t ret;
	
	PICR = ICR_IUE | ICR_SCLE;
	PIDBR = PMIC_SLAVE_ADDR<<1;
	PICR |= ICR_START | ICR_TB;
	while(PICR & ICR_TB);
	
	//actually send the address terminated with a STOP
	PIDBR = address;
	PICR &= ~ICR_START;
	PICR |= ICR_STOP | ICR_TB;
	while(PICR & ICR_TB);
	PICR &= ~ICR_STOP;
	
	//actually request the read of the data
	PIDBR = PMIC_SLAVE_ADDR << 1 | 1;
	PICR |= ICR_START | ICR_TB;
	while(PICR & ICR_TB);
	PICR &= ~ICR_START;
	
	PICR |= ICR_STOP | ICR_ACKNAK| ICR_TB;
	while(PICR & ICR_TB);
	ret = PIDBR;
	
	PICR &= ~(ICR_STOP|ICR_ACKNAK|ICR_IUE | ICR_SCLE);
	
	PCFR = PCFR_PI2C_EN | PCFR_FVC | PCFR_DC_EN;
	return ret;
}

static void initInterrupts() {
	uint8_t i;
	
	DISABLE_INTERRUPTS();
	
	ICMR = 0;
	ICMR2 = 0;
	
	// set DIM bit, so that only unmasked interrupts cause an interrupt to the processor when CPU is in idle mode
	ICCR = 1;
	for(i=0; i <= 39; ++i) {
		IPR(i) = i | IPR_VAL;
	}
	
	// at reboot all interrupts are IRQ Interrupts so ICLR is not need to be set, all interrupts cause IRQ interrupts
	// enable intterupts in the CPSR
	ENABLE_INTERRUPTS();
}

void hardwareInit() {
	initPMIC ();
	cpufreqInit();
	initInterrupts ();
}

